<?php

namespace Lavalite\Testerlite\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Lavalite\Testerlite\Forms\Test as TestForm;
use Lavalite\Testerlite\Http\Requests\TestRequest;
use Lavalite\Testerlite\Http\Resources\TestResource;
use Lavalite\Testerlite\Http\Resources\TestsCollection;
use Lavalite\Testerlite\Models\Test;
use Lavalite\Testerlite\Scopes\TestResourceScope;


/**
 * Resource controller class for test.
 */
class TestResourceController extends BaseController
{

    /**
     * Initialize test resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = TestForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('lavalite.testerlite.modules'), 'testerlite', guard_url('testerlite'));
    }

    /**
     * Display a list of test.
     *
     * @return Response
     */
    public function index(TestRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Test::pushScope(new RequestScope())
            ->pushScope(new TestResourceScope())
            ->simplePaginate($pageLimit);

        $data = new TestsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('testerlite::test.names'))
            ->view('testerlite::test.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display test.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function show(TestRequest $request, Test $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('testerlite::test.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('testerlite::test.show')
            ->output();

    }

    /**
     * Show the form for creating a new test.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TestRequest $request, Test $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('testerlite::test.name'))
            ->view('testerlite::test.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new test.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TestRequest $request, Test $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new TestResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('testerlite::test.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('testerlite/test/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/testerlite/test'))
                ->redirect();
        }

    }

    /**
     * Show test for editing.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function edit(TestRequest $request, Test $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('testerlite::test.name'))
            ->view('testerlite::test.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the test.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function update(TestRequest $request, Test $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new TestResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('testerlite::test.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('testerlite/test/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('testerlite/test/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the test.
     *
     * @param Model   $test
     *
     * @return Response
     */
    public function destroy(TestRequest $request, Test $model)
    {
        try {
            $model->delete();
            $data = new TestResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('testerlite::test.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('testerlite/test/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('testerlite/test/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
